#!/bin/bash
# Updates HomeControl
VERSIONURL=https://server47.de/homecontrol/interface.php?request=showAvailableVersion
CHECKSUMURL=https://server47.de/homecontrol/interface.php?request=checksum
ZIPURL=https://server47.de/homecontrol/interface.php?request=getLatestVersion

HCFOLDER=/home/pi/HomeControl
LOGFILE="$HCFOLDER/HomeControlLogFile.txt"
TMPFILE=/tmp/homecontrolversion.txt
TMPCHK=/tmp/homecontrolchecksum.txt
wget "$VERSIONURL" -O "$TMPFILE" --ca-certificate="$HCFOLDER"/ca_cacert.pem
AVAILABLEVERSION=$(cat "$TMPFILE")
INSTALLEDVERSION=$(cat version.txt)
LOCALARCHIVE=/home/pi/homecontrol.tar.gz
OLDFILE=HomeControl.jar.bak
NEWFILE=HomeControl.jar

function downloadUpdate()
{
	echo "Downloading newest version..."
	wget "$ZIPURL" -O "$LOCALARCHIVE" --ca-certificate="$HCFOLDER"/ca_cacert.pem

	if [ "$?" -eq "0" ]
	then		
		echo "Download complete."
	else
		echo "Error downloading package from $ZIPURL"
		exit 3
	fi
}

function verifyChecksums()
{
	CHECKSUMLOCAL=$(md5sum "$LOCALARCHIVE" |awk '{print $1}')
	wget "$CHECKSUMURL" -O "$TMPCHK" --ca-certificate="$HCFOLDER"/ca_cacert.pem
	NEWCHECKSUMFROMSERVER=$(cat "$TMPCHK")
	rm "$TMPCHK"

	echo "Checking checksums..."
	echo "============================="
	echo "Remote checksum: $NEWCHECKSUMFROMSERVER"
	echo "Local checksum:  $CHECKSUMLOCAL"
	echo "============================="

	if [ "$CHECKSUMLOCAL" = "$NEWCHECKSUMFROMSERVER" ]
	then
		echo "Checksums match!"
	else
		echo "Wrong checksum! Try to restart script."
		exit 2
	fi
}

function stopDaemon()
{
	# Stop daemon if running
	echo "Stopping daemon if running..."
	sudo systemctl stop homecontrol.service

	if [ "$?" = "0" ]
	then
		mv "$HCFOLDER"/HomeControl.jar "$HCFOLDER"/"$OLDFILE"
	else
		echo "Error stopping daemon."
		exit 5
	fi
}

function updateDatabase()
{
	cd "$HCFOLDER"
	echo "Updating database..."

	java -jar HomeControl.jar -updateDatabase

	if [ "$?" -eq "0" ]
	then
		echo "Database update complete."

		echo "Starting daemon..."

		#sudo service homecontrol start
	else
		echo "Error updating database."
		exit 1
	fi
}

function rollback()
{
	echo "Rolling back..."

	mv "$HCFOLDER"/"$OLDFILE" "$HCFOLDER"/"$NEWFILE"

	if [ "$?" -eq "0" ]
	then
		echo "Error unpacking new archive. The old version has been restored."
	else
		echo "Error rolling back old version."
	fi
}

echo "============================="
echo "Available version: $AVAILABLEVERSION"
echo "Installed version: $INSTALLEDVERSION"
echo "============================="

if [ $AVAILABLEVERSION -gt $INSTALLEDVERSION ]
then
	echo "Update available"
	rm "$TMPFILE"

	downloadUpdate
		
	verifyChecksums

	stopDaemon

	if [ "$?" -eq "0" ]
	then
		# Unpack downloaded file
		echo "Unpacking archive..."
		tar xzvf "$LOCALARCHIVE" -C /home/pi
		
		if [ "$?" -eq "0" ]
		then
			updateDatabase

			# Update possibly changed start-stop-script
			sudo systemctl daemon-reload
			sudo systemctl enable homecontrol.service
			sudo systemctl start homecontrol.service

			if [ "$?" -eq "0" ]
			then
				echo "==================================================="
				echo "           Update finished successfully!"
				echo "==================================================="
			else
				echo "================================================================"
				echo "Daemon could not be started, but update should be successfull."
				echo "================================================================"
			fi
		else
			rollback
		fi
	else
		echo "Error stopping daemon"
	fi
	

	exit 1
else
	echo "No update available. Latest version already installed."
	exit 0
fi
